package dev.jansel.platinum

import io.ktor.client.*
import io.ktor.client.call.*
import io.ktor.client.engine.cio.*
import io.ktor.client.plugins.*
import io.ktor.client.plugins.contentnegotiation.*
import io.ktor.client.request.*
import io.ktor.http.*
import io.ktor.serialization.kotlinx.json.*
import io.papermc.paper.command.brigadier.Commands
import io.papermc.paper.plugin.lifecycle.event.LifecycleEventManager
import io.papermc.paper.plugin.lifecycle.event.types.LifecycleEvents
import kotlinx.coroutines.runBlocking
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.Json
import kotlinx.serialization.json.buildJsonArray
import net.kyori.adventure.text.Component
import net.kyori.adventure.text.format.TextColor
import org.bukkit.Server
import org.bukkit.command.CommandExecutor
import org.bukkit.command.PluginCommand
import org.bukkit.command.TabCompleter
import org.bukkit.plugin.Plugin
import org.bukkit.plugin.PluginManager
import org.bukkit.plugin.java.JavaPlugin
import org.bukkit.scheduler.BukkitScheduler
import java.util.Properties
import java.util.logging.Logger

@Suppress("unused")
class Platinum : JavaPlugin() {
    companion object {
        lateinit var instance: Platinum
        lateinit var pluginManager: PluginManager
        lateinit var logger: Logger
        lateinit var server: Server
        lateinit var props: Properties
        lateinit var version: String
        lateinit var hash: String
        lateinit var webClient: HttpClient
    }



    override fun onEnable() {

    }

    override fun onLoad() {
        props = Properties()
        props.load(getResource("build-vars.properties"))
        version = props.getProperty("version")
        hash = props.getProperty("gitCommit")
        logger.info("Loading Platinum v$version")
        webClient = HttpClient(CIO) {
            install(ContentNegotiation) {
                json(Json { ignoreUnknownKeys = true })
            }
            install(UserAgent) {
                agent = "https://git.jansel.dev/jansel/Platinum v$version(jansel@jansel.moe)"
            }
        }
        if (version.contains("SNAPSHOT")) {
            logger.info("Local Git Commit: $hash")
            logger.info("Getting remote Git Commit...")
            runBlocking {
                val res = webClient.get("https://git.jansel.dev/") {
                    url {
                        path("/api/v1/repos/jreimers/platinum/commits")
                        parameters.append("limit", "1")
                    }
                }
                val apires: List<CommitApiResponse> = res.body()
                logger.info("Remote hash: ${apires[0].sha}")
                if (apires[0].sha == hash) {
                    componentLogger.info(Component.text("Commit Hashes match, ur on the latest snapshot version",
                        TextColor.color(0,255,0)))
                } else {
                    componentLogger.info(Component.text("Commit Hashes mismatch, either you\'re using a commit that isnt pushed yet or you\'re out of date",
                        TextColor.color(255,0,0)))
                }
            }
        }
        Companion.server = server
        pluginManager = server.pluginManager
        instance = this
    }

    override fun onDisable() {
        // Plugin shutdown logic
    }

    @Serializable
    data class CommitApiResponse(
        val sha: String
    )
}
